from _typeshed import Incomplete

from openpyxl import _VisibilityType
from openpyxl.worksheet.worksheet import Worksheet

def read_dimension(source): ...

class ReadOnlyWorksheet:
    cell = Worksheet.cell
    iter_rows = Worksheet.iter_rows
    # Same as Worksheet.values
    @property
    def values(self): ...
    # Same as Worksheet.rows
    @property
    def rows(self): ...
    __getitem__ = Worksheet.__getitem__
    __iter__ = Worksheet.__iter__
    parent: Incomplete
    title: str
    sheet_state: _VisibilityType
    def __init__(self, parent_workbook, title: str, worksheet_path, shared_strings) -> None: ...
    def calculate_dimension(self, force: bool = False): ...
    def reset_dimensions(self) -> None: ...
    @property
    def min_row(self): ...
    @property
    def max_row(self): ...
    @property
    def min_column(self): ...
    @property
    def max_column(self): ...
